{*****************************************************************
*                                                                *
*        Week Number Calculator                                  *
*                                                                *
*        Copyright 1999 Complete Git Software                    *
*                                                                *
*        Author: Aubrey Dean                                     *
*                                                                *
*  You are free to use any part of this code for any application *
*  Email comments & questions to aubs@completeg.freeserve.co.uk  *
*                                                                *
******************************************************************}

unit weekNum;

{ISO standard 8601:1988 - (European Standard EN 28601).
Contains information on how to format date/time representations, the leap years, etc.,
It states that a week is identified by its number in a given year.
A week begins with a Monday (day 1) and ends with a Sunday (day 7).
The first week of a year is the one which includes the first Thursday (day 4),
or equivalently the one which includes January 4.

In other words, the first week of a new year is the week that has the majority of its days in the new year.

Note:

i)  Week 01 might contain days from the previous year,
ii) The week prior to week 01 of a year is the last week (52 or 53) of the previous year
(even if it contains days from the new year).}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Calendar, ComCtrls, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Calendar1: TCalendar;
    Panel1: TPanel;
    Label1: TLabel;
    edtYear: TEdit;
    udYear: TUpDown;
    Label2: TLabel;
    edtMonth: TEdit;
    udMonth: TUpDown;
    procedure Calendar1Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure edtYearChange(Sender: TObject);
    procedure edtMonthChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


//gets a datecode. Returns year and weeknumber in format: YYWW
function getWeekNumber(today: Tdatetime): string;

{dayOfWeek function returns integer 1..7 equivalent to Sunday..Saturday.
ISO 8601 weeks start with Monday and the first week of a year is the one which
includes the first Thursday - Fiddle takes care of all this}

const Fiddle : array[1..7] of Byte = (6,7,8,9,10,4,5);

var
	present, startOfYear: Tdatetime;
	firstDayOfYear, weekNumber, numberOfDays: integer;
	year, month, day: word;
	YearNumber: string;

begin
	present:= trunc(today); //truncate to remove hours, mins and secs
	decodeDate(present, year, month, day); //decode to find year
	startOfYear:= encodeDate(year, 1, 1);  //encode 1st Jan of the year

  //find what day of week 1st Jan is, then add days according to rule
	firstDayOfYear:= Fiddle[dayOfWeek(startOfYear)];

	//calc number of days since beginning of year + additional according to rule
	numberOfDays:= trunc(present - startOfYear) + firstDayOfYear;

	//calc number of weeks
	weekNumber:= trunc(numberOfDays / 7);

	//Format year, needed to prevent millenium bug and keep the Fluffy Spangle happy
	YearNumber:= formatDateTime('yyyy',present);

	YearNumber:= YearNumber + 'W';

	if weekNumber < 10 then
    YearNumber:= YearNumber + '0';//add leading zero for week

	//create datecode string
	result:= YearNumber + inttostr(weekNumber);

  if weekNumber = 0 then //recursive call for year begin/end...
    //see if previous year end was week 52 or 53
    result:= getWeekNumber(encodeDate(year - 1, 12, 31))

  else if weekNumber = 53 then
    //if 31st December less than Thursday then must be week 01 of next year
    if dayOfWeek(encodeDate(year, 12, 31)) < 5 then
    begin
      YearNumber:= formatDateTime('yyyy',encodeDate(year + 1, 1, 1));
      result:= YearNumber + 'W01';
    end;

end;


procedure TForm1.Calendar1Change(Sender: TObject);
begin
  with Calendar1 do
    Caption:= getWeekNumber(encodeDate(Year, Month, Day));
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  udYear.Position:= Calendar1.Year;
  udMonth.Position:= Calendar1.Month;
end;

procedure TForm1.edtYearChange(Sender: TObject);
begin
  Calendar1.Year:= udYear.Position;
end;

procedure TForm1.edtMonthChange(Sender: TObject);
begin
  Calendar1.Month:= udMonth.Position;
end;

end.
